/****************************************************
*
* Name: M2's Gib-Slay
* Version: 1.8
* Date: 10-16-2007
* E-mail: wesley.phantomtrilogy3@gmail.com
* Website: www.killmachine.com
*
* Description:
* - This plugin will slay a player which results in a mess of guts, bones, and blood.
*
* Commands:
* - amx_gibslay <authid, nick, @team, @all or #userid> [REQUIRES ADMIN_SLAY FLAG]
* - amx_gibslay_nk <authid, nick, @team, @all or #userid> [REQUIRES ADMIN_SLAY FLAG]
* - amx_gibslay_aim <target in crosshair> [REQUIRES ADMIN_SLAY FLAG]
* - amx_gibslay_nk_aim <target in crosshair> [REQUIRES ADMIN_SLAY FLAG]
* 
* Command Details:
* - amx_gibslay will gib-slay the target player.
* - amx_gibslay_nk (nk = no kill) will gib-slay the target, but the target is not harmed.
* - amx_gibslay_aim will gib-slay the player in your crosshair.
* - amx_gibslay_nk_aim will execute the nasty gib-slaying effect on the player in your crosshair.
*
* CVARs:
* - m2_gs_count <1-500> - number of gibs (default 25)
* - m2_gs_velocity <1-9999> - velocity of gibs (default 250)
* - m2_gs_life <1-999> - how long the gibs stay on the floor (default 30)
* - m2_gs_gibtype <1|2> - what kind of gibs are used; 1 = human, 2 = alien (default 1)
* - m2_gs_immunity <1|0> - if this is 1, players with immunity are not harmed (default 1)
* - m2_gs_enabled <1|0> - enables/disables gib-slaying (default 1)
* - m2_gs_customsound <1|0> - use a custom sound if the file exists; 1 = on, 2 = off (default 1)
* - m2_gs_usesound <1|0> - use sound when gib-slaying; 1 = on, 2 = off (default 1)
* - m2_gs_logging <1|0> - enables/disables logging of gib-slaying; 1 = on, 2 = off (default 1)
*
* Notes:
* - Be nice to players with lower-end machines; they can't always handle large amounts of entities.
* - Valid teams for TFC: @blue, @red, @yellow, @green
* - Valid teams for CS: @CT, @T
*
* Special Thanks (and Credits):
* - KiLL Machine Guild -[KM]- (www.killmachine.com)
* - Special thanks to |SPANKIN|SamuraiBarbi-[KM]- for helping me test this plugin.
* - AMX Mod X community for awesome support.
* - -={*UGC*}=-$ilent$niper for pointing out the visible spectator bug.
* - Ultimate Gore Enhancement for blood decals.
* - Zenith77 for pointing me to a thread with decal ID's.
*
* Known Bugs:
* - If gib-slaying a player the moment they respawn, their body will not be set invisible.
* - Using the _aim commands may not work on the first try, and repeatedly pressing the button
* which it's bound to will not make it any more efficient.
*
* Changelog:
* - 1.0: Initial release.
* - 1.1: Added control over the amount of gibs and the velocity; Cleaned up code.
* - 1.2: Added control to enable/disable the plugin; Cleaned up code.
* - 1.3: Added control to the life of the gibs and how much blood is created; Revised permissions.
* - 1.4: Code cleanup; Removed logging; Removed useless functions; Removed redundant code.
* - 1.5: Added alien gibs; Change CVAR "m2_gs_gibtype" to "2" for alien gibs; Removed a glitch.
* where spectators would be visible while this plugin was enabled; Extra blood effect for TFC when
* using the human gib-slay type; Removed CVAR "m2_gs_bloodlevel", as it was unnecessary and could
* easily be abused.
* - 1.6: Added ability to gib-slay multiple targets at once; Added immunity control; Added ability
* to change between the custom sound and default sound (only if the custom sound exists, or else the
* default sound will be used no matter what).
* - 1.7: Added a new way to gib-slay by using the new _aim commands to slay the player in one's
* crosshair.
* - 1.8: Added logging again.
*
****************************************************/

// Includes
#include <amxmodx>
#include <amxmisc>
#include <engine>

// Defines
#define PLUGIN "M2's Gib-Slay"
#define AUTHOR "M249-M4A1"
#define VERSION "1.8"
#define M2GIBSOUND "sound/m249-m4a1/m2bodysplat.wav"

// Variables
new bool:ModTFC = false
new bool:SoundExists = false
new blooddecal[2], humangibs, aliengibs, blooddrop, bloodspray
new gCount, gVelocity, gEnabled, gLife, gGibType, gObeyImmunity
new gCustomSound, gUseSound, gLogging

public plugin_init()
{
	// Basic plugin registration
	register_plugin(PLUGIN, VERSION, AUTHOR)
	register_concmd("amx_gibslay", "GibSlay", ADMIN_SLAY, "<authid, nick, @team, @all or #userid> - gib-slay a player")
	register_concmd("amx_gibslay_nk", "GibSlay", ADMIN_SLAY, "<authid, nick, @team, @all or #userid> - gib-slay a player (no kill)")
	register_concmd("amx_gibslay_aim", "GibSlayAim", ADMIN_SLAY, "<target in crosshair>")
	register_concmd("amx_gibslay_nk_aim", "GibSlayAim", ADMIN_SLAY, "<target in crosshair> (no kill)")
	register_event("ResetHUD", "HookResetHUD", "b")
	
	// CVARs
	gCount = register_cvar("m2_gs_count", "25")
	gVelocity = register_cvar("m2_gs_velocity", "250")
	gEnabled = register_cvar("m2_gs_enabled", "1")
	gLife = register_cvar("m2_gs_life", "30")
	gGibType = register_cvar("m2_gs_gibtype", "1")
	gObeyImmunity = register_cvar("m2_gs_immunity", "1")
	gCustomSound = register_cvar("m2_gs_customsound", "1")
	gUseSound = register_cvar("m2_gs_usesound", "1")
	gLogging = register_cvar("m2_gs_logging", "1")
}

public plugin_cfg()
{
	// Check for TFC
	new mod[16]
	get_modname(mod, 31)
	if (equali(mod, "tfc"))
	{
		blooddecal = {208, 209}
		ModTFC = true
	}
	else
		ModTFC = false
}

public plugin_precache()
{
	// Precache some things
	humangibs = precache_model("models/hgibs.mdl")
	aliengibs = precache_model("models/agibs.mdl")
	blooddrop = precache_model("sprites/blood.spr")
	bloodspray = precache_model("sprites/bloodspray.spr")
	
	// Check if the custom sound file exists
	if (file_exists(M2GIBSOUND))
	{
		SoundExists = true
		precache_sound("m249-m4a1/m2bodysplat.wav")
	}
	else
	{
		SoundExists = false
		server_print("[AMXX] ERROR: Missing gib-slay sound file: '%s'. Plugin will use default gib sound.", M2GIBSOUND)
		server_print("[AMXX] Add the file and restart the map to use the custom sound.")
	}
	
	// This is the most basic sound, and must be precached
	precache_sound("common/bodysplat.wav")
}

public GibSlay(id, level, cid)
{
	// Check if the plugin is disabled
	if (!CheckEnabled())
	{
		console_print(id, "[AMXX] Sorry, gib-slaying is currently disabled!")
		return PLUGIN_HANDLED
	}
	
	// Check if the user has access
	if (!cmd_access(id, level, cid, 2))
		return PLUGIN_HANDLED
	
	// Read the name of the target player
	new arg[32], CheckNK[32], victim, team = 0
	new bool:doKill = false, bool:isMultiSlay = false, bool:doLog = false
	read_argv(1, arg, 31)
	
	// Check if the user is doing the no-kill type
	read_argv(0, CheckNK, 32)
	if(equali(CheckNK[12] , "n", 1))
		doKill = false
	else
		doKill = true
	
	// Check if the user wants to gib-slay mulitple targets
	if (arg[0] == '@')
	{
		isMultiSlay = true
		new players[32], totalplayers, player
		get_players(players, totalplayers)
		
		// Get gib-slay arguments
		if(equali(arg[1], "all"))
		{
			// Immunity check
			for (new i = 0; i < totalplayers; i++)
			{
				new name[32]
				player = players[i]
				get_user_name(player, name, 31)
				
				if (get_pcvar_num(gObeyImmunity) > 0)
				{
					if (get_user_flags(player) & ADMIN_IMMUNITY)
						console_print(id, "[AMXX] Skipping gib-slay on user '%s' due to immunity.", name)
					else
						GibSlayUser(player, doKill, isMultiSlay)
				}
				else
				{
					// Slay everyone
					GibSlayUser(player, doKill, isMultiSlay)
				}
			}
			// Log
			GibSlayLogging(id, player, doKill, isMultiSlay, team)
			return PLUGIN_HANDLED
		}
		else if (equali(arg[1], "BLUE") || equali(arg[1], "T") || equali(arg[1], "1"))
			team = 1
		else if (equali(arg[1], "RED") || equali(arg[1], "CT") || equali(arg[1], "2"))
			team = 2
		else if(equali(arg[1], "YELLOW") || equali(arg[1], "3"))
			team = 3
		else if(equali(arg[1], "GREEN") || equali(arg[1], "4"))
			team = 4
		for (new i = 0; i < totalplayers; i++)
		{
			new name[32]
			player = players[i]
			get_user_name(player, name, 31)
			
			if (get_user_team(player) == team)
			{
				if (get_pcvar_num(gObeyImmunity) > 0)
				{
					// Check for immunity
					if (get_user_flags(player) & ADMIN_IMMUNITY)
						console_print(id, "[AMXX] Skipping gib-slay on user '%s' due to immunity.", name)
					else
						GibSlayUser(player, doKill, isMultiSlay)
				}
				else
				{
					// Slay the player if they are on the given team
					GibSlayUser(player, doKill, isMultiSlay)
				}
				doLog = true
			}
		}
		// Log the event only if gib-slaying actually happened
		if (doLog)
			GibSlayLogging(id, player, doKill, isMultiSlay, team)	
		return PLUGIN_HANDLED
	}
	else
	{
		// Get the target player
		if (get_pcvar_num(gObeyImmunity) > 0)
			victim = cmd_target(id, arg, 5)
		else
			victim = cmd_target(id, arg, 7)
		
		// Check if the player exists
		if (!victim)
			return PLUGIN_HANDLED
		
		// Create the nasty explosion of body parts and blood
		GibSlayUser(victim, doKill, isMultiSlay)
		
		// If logging is enabled, log the event
		GibSlayLogging(id, victim, doKill, isMultiSlay, team)
	}
	return PLUGIN_HANDLED
}

public GibSlayAim(id, level, cid)
{
	// Check if the plugin is disabled
	if (!CheckEnabled())
	{
		client_print(id, print_chat, "[AMXX] Sorry, gib-slaying is currently disabled!")
		return PLUGIN_HANDLED
	}
	
	// Check if the user has access
	if (!cmd_access(id, level, cid, 1))
		return PLUGIN_HANDLED
	
	new victim, body
	get_user_aiming(id, victim, body, 9999)
	
	if(is_valid_ent(victim))
	{
		new class[33]
		entity_get_string(victim, EV_SZ_classname, class, 32)
		
		if(equal(class, "player") && is_user_alive(victim))
		{
			new CheckNK[32], bool:doKill = false
			
			// Check if the user is doing the no-kill type
			read_argv(0, CheckNK, 32)
			if(equali(CheckNK[12] , "n", 1))
				doKill = false
			else
				doKill = true
			
			// Check for immunity
			if (get_pcvar_num(gObeyImmunity) > 0)
			{
				new name[32]
				get_user_name(victim, name, 31)
				
				if (get_user_flags(victim) & ADMIN_IMMUNITY)
					client_print(id, print_chat, "[AMXX] You cannot gib-slay '%s' because they have immunity.", name)
				else
					GibSlayUser(victim, doKill, false)
			}
			else
			{
				// Slay the target
				GibSlayUser(victim, doKill, false)
			}
			// Log
			GibSlayLogging(id, victim, doKill, false, 0)
		}
	}
	return PLUGIN_HANDLED
}

public GibSlayUser(player, bool:doKill, bool:isMultiSlay)
{
	if (is_user_alive(player))
	{
		// Get the target origin
		new origin[3]
		get_user_origin(player, origin)
		
		// Check for a valid number of gibs before creating them
		if (get_pcvar_num(gCount) > 500)
			set_pcvar_num(gCount, 500)
		
		// Create a bunch of gibs
		message_begin(MSG_BROADCAST,SVC_TEMPENTITY,{0,0,0},player) 
		write_byte(107)
		write_coord(origin[0])
		write_coord(origin[1]) 
		write_coord(origin[2] + 10) 
		write_coord(get_pcvar_num(gVelocity))
		switch(get_pcvar_num(gGibType))
		{
			case 2: write_short(aliengibs)
				default: write_short(humangibs)
		}
		// Check for multi-gib-slay
		if (!isMultiSlay)
		{
			write_short(get_pcvar_num(gCount))
			write_byte(get_pcvar_num(gLife))
		}
		else
		{
			write_short(2)
			write_byte(10)
		}
		message_end()
		
		// Create some blood effects
		if (!isMultiSlay)
		{
			for (new i = 0; i < 5; i++)
				BloodEffects(origin)
		}
		else
			BloodEffects(origin)
		
		// Play a nasty sound if enabled
		if (get_pcvar_num(gUseSound) > 0)
		{
			if (!SoundExists)
				emit_sound(player, CHAN_ITEM, "common/bodysplat.wav", 1.0, ATTN_NORM, 0, PITCH_NORM)
			else
			{
				if (get_pcvar_num(gCustomSound) > 0)
					emit_sound(player, CHAN_ITEM, "m249-m4a1/m2bodysplat.wav", 1.0, ATTN_NORM, 0, PITCH_NORM)
				else
					emit_sound(player, CHAN_ITEM, "common/bodysplat.wav", 1.0, ATTN_NORM, 0, PITCH_NORM)
			}
		}
		// If this is not the no-kill type, kill the user
		if (doKill == true)
		{
			set_entity_visibility(player, 0)
			user_kill(player)
		}
	}
}

public BloodEffects(origin[3])
{
	// Victim has a stream of blood
	message_begin(MSG_BROADCAST,SVC_TEMPENTITY)
	write_byte(101)
	write_coord(origin[0])
	write_coord(origin[1])
	write_coord(origin[2] + 15)
	write_coord(random_num(-360, 360))
	write_coord(random_num(-360, 360))
	write_coord(-10)
	switch(get_pcvar_num(gGibType))
	{
		case 2: write_byte(194)
			default: write_byte(70)
	}
	write_byte(random_num(55,110))
	message_end()
	
	// Victim has an explosion of blood
	message_begin(MSG_BROADCAST,SVC_TEMPENTITY)
	write_byte(115)
	write_coord(origin[0])
	write_coord(origin[1])
	write_coord(origin[2] + 15)
	write_short(bloodspray)
	write_short(blooddrop)
	switch(get_pcvar_num(gGibType))
	{
		case 2: write_byte(194)
			default: write_byte(248)
	}
	write_byte(20)
	message_end()
	
	// If the game is TFC and gib-slay is set to human gibs, create extra
	// blood decals on the floor
	if (ModTFC == true && get_pcvar_num(gGibType) != 2)
	{
		// Splash blood around
		message_begin(MSG_BROADCAST,SVC_TEMPENTITY)
		write_byte(116)
		write_coord(origin[0] + random_num(-30, 30))
		write_coord(origin[1] + random_num(-30, 30))
		write_coord(origin[2] - 35)
		write_byte(blooddecal[random_num(0,1)])
		message_end()
	}
}

public GibSlayLogging(id, victim, doKill, isMultiSlay, team)
{
	if (get_pcvar_num(gLogging) > 0)
	{
		// Log gib-slay events (altered directly from admincmd.sma to maintain consistent)
		new authid[32], name2[32], authid2[32], name[32]
		
		get_user_authid(id, authid, 31)
		get_user_name(id, name, 31)
		get_user_authid(victim, authid2, 31)
		get_user_name(victim, name2, 31)
		
		if (!doKill)
		{
			if (!isMultiSlay)
				log_amx("Cmd: ^"%s<%d><%s><>^" gib-slay (no-kill) ^"%s<%d><%s><>^"", name, get_user_userid(id), authid, name2, get_user_userid(victim), authid2)
			else if (isMultiSlay && team != 0)
				log_amx("Cmd: ^"%s<%d><%s><>^" gib-slay (no-kill) ^"Team #%i^"", name, get_user_userid(id), authid, team)
			else if (isMultiSlay && team == 0)
				log_amx("Cmd: ^"%s<%d><%s><>^" gib-slay (no-kill) ^"ALL^"", name, get_user_userid(id), authid)
		}
		else
		{
			if (!isMultiSlay)
			{
				// Only display gib-slay activity if the gib-slay was lethal
				switch (get_cvar_num("amx_show_activity"))
				{
					case 2: client_print(0, print_chat, "ADMIN %s: gib-slay %s", name, name2)
						case 1: client_print(0, print_chat, "ADMIN: Gib-slay %s", name2)
					}
				log_amx("Cmd: ^"%s<%d><%s><>^" gib-slay ^"%s<%d><%s><>^"", name, get_user_userid(id), authid, name2, get_user_userid(victim), authid2)
			}
			else if (isMultiSlay && team != 0)
			{
				// Only display gib-slay activity if the gib-slay was lethal
				switch (get_cvar_num("amx_show_activity"))
				{
					case 2: client_print(0, print_chat, "ADMIN %s: gib-slay Team #%i", name, team)
						case 1: client_print(0, print_chat, "ADMIN: Gib-slay Team #%i", team)
					}
				log_amx("Cmd: ^"%s<%d><%s><>^" gib-slay ^"Team #%i^"", name, get_user_userid(id), authid, team)
			}
			else if (isMultiSlay && team == 0)
			{
				// Only display gib-slay activity if the gib-slay was lethal
				switch (get_cvar_num("amx_show_activity"))
				{
					case 2: client_print(0, print_chat, "ADMIN %s: gib-slay everyone", name)
						case 1: client_print(0, print_chat, "ADMIN: Gib-slay everyone")
					}
				log_amx("Cmd: ^"%s<%d><%s><>^" gib-slay ^"ALL^"", name, get_user_userid(id), authid)
			}
		}
	}
}

public HookResetHUD(player)
{
	// Check if the player is a spectator by using set_task
	// because an ordinary if statement would execute too fast
	set_task(0.1, "FixSpectator", player)
	
}

public FixSpectator(player)
{
	// If not a spectator, set the player visible again
	if (get_user_team(player) != 0)
		set_entity_visibility(player, 1)
}

public CheckEnabled()
{
	// Return true of the plugin is enabled, else, return false
	if (get_pcvar_num(gEnabled) > 0)
		return true
	else
		return false
	return PLUGIN_CONTINUE
}
